@@ -4,7 +4,7 @@ from __future__ import annotations
 import asyncio
 from dataclasses import InitVar, dataclass, field
 from datetime import datetime, timedelta
-from typing import TYPE_CHECKING, Any, Callable, Dict, cast
+from typing import TYPE_CHECKING, Any, Callable, Dict, Final, cast
 
 from aiohttp import ClientWebSocketResponse, WSMsgType
 from aiohttp.client_exceptions import (
@@ -22,6 +22,7 @@ from simplipy.errors import (
     InvalidMessageError,
     NotConnectedError,
 )
+from simplipy.util import schedule_callback
 from simplipy.util.dt import utc_from_timestamp
 
 if TYPE_CHECKING:
@@ -31,34 +32,34 @@ WEBSOCKET_SERVER_URL = "wss://socketlink.prd.aser.simplisafe.com"
 
 DEFAULT_WATCHDOG_TIMEOUT = timedelta(minutes=5)
 
-EVENT_ALARM_CANCELED = "alarm_canceled"
-EVENT_ALARM_TRIGGERED = "alarm_triggered"
-EVENT_ARMED_AWAY = "armed_away"
-EVENT_ARMED_AWAY_BY_KEYPAD = "armed_away_by_keypad"
-EVENT_ARMED_AWAY_BY_REMOTE = "armed_away_by_remote"
-EVENT_ARMED_HOME = "armed_home"
-EVENT_AUTOMATIC_TEST = "automatic_test"
-EVENT_AWAY_EXIT_DELAY_BY_KEYPAD = "away_exit_delay_by_keypad"
-EVENT_AWAY_EXIT_DELAY_BY_REMOTE = "away_exit_delay_by_remote"
-EVENT_CAMERA_MOTION_DETECTED = "camera_motion_detected"
-EVENT_CONNECTION_LOST = "connection_lost"
-EVENT_CONNECTION_RESTORED = "connection_restored"
-EVENT_DISARMED_BY_MASTER_PIN = "disarmed_by_master_pin"
-EVENT_DISARMED_BY_REMOTE = "disarmed_by_remote"
-EVENT_DOORBELL_DETECTED = "doorbell_detected"
-EVENT_DEVICE_TEST = "device_test"
-EVENT_ENTRY_DELAY = "entry_delay"
-EVENT_HOME_EXIT_DELAY = "home_exit_delay"
-EVENT_LOCK_ERROR = "lock_error"
-EVENT_LOCK_LOCKED = "lock_locked"
-EVENT_LOCK_UNLOCKED = "lock_unlocked"
-EVENT_POWER_OUTAGE = "power_outage"
-EVENT_POWER_RESTORED = "power_restored"
-EVENT_SECRET_ALERT_TRIGGERED = "secret_alert_triggered"
-EVENT_SENSOR_NOT_RESPONDING = "sensor_not_responding"
-EVENT_SENSOR_PAIRED_AND_NAMED = "sensor_paired_and_named"
-EVENT_SENSOR_RESTORED = "sensor_restored"
-EVENT_USER_INITIATED_TEST = "user_initiated_test"
+EVENT_ALARM_CANCELED: Final = "alarm_canceled"
+EVENT_ALARM_TRIGGERED: Final = "alarm_triggered"
+EVENT_ARMED_AWAY: Final = "armed_away"
+EVENT_ARMED_AWAY_BY_KEYPAD: Final = "armed_away_by_keypad"
+EVENT_ARMED_AWAY_BY_REMOTE: Final = "armed_away_by_remote"
+EVENT_ARMED_HOME: Final = "armed_home"
+EVENT_AUTOMATIC_TEST: Final = "automatic_test"
+EVENT_AWAY_EXIT_DELAY_BY_KEYPAD: Final = "away_exit_delay_by_keypad"
+EVENT_AWAY_EXIT_DELAY_BY_REMOTE: Final = "away_exit_delay_by_remote"
+EVENT_CAMERA_MOTION_DETECTED: Final = "camera_motion_detected"
+EVENT_CONNECTION_LOST: Final = "connection_lost"
+EVENT_CONNECTION_RESTORED: Final = "connection_restored"
+EVENT_DISARMED_BY_MASTER_PIN: Final = "disarmed_by_master_pin"
+EVENT_DISARMED_BY_REMOTE: Final = "disarmed_by_remote"
+EVENT_DOORBELL_DETECTED: Final = "doorbell_detected"
+EVENT_DEVICE_TEST: Final = "device_test"
+EVENT_ENTRY_DELAY: Final = "entry_delay"
+EVENT_HOME_EXIT_DELAY: Final = "home_exit_delay"
+EVENT_LOCK_ERROR: Final = "lock_error"
+EVENT_LOCK_LOCKED: Final = "lock_locked"
+EVENT_LOCK_UNLOCKED: Final = "lock_unlocked"
+EVENT_POWER_OUTAGE: Final = "power_outage"
+EVENT_POWER_RESTORED: Final = "power_restored"
+EVENT_SECRET_ALERT_TRIGGERED: Final = "secret_alert_triggered"
+EVENT_SENSOR_NOT_RESPONDING: Final = "sensor_not_responding"
+EVENT_SENSOR_PAIRED_AND_NAMED: Final = "sensor_paired_and_named"
+EVENT_SENSOR_RESTORED: Final = "sensor_restored"
+EVENT_USER_INITIATED_TEST: Final = "user_initiated_test"
 
 EVENT_MAPPING = {
     1110: EVENT_ALARM_TRIGGERED,
@@ -116,11 +117,7 @@ class Watchdog:
     def _on_expire(self) -> None:
         """Log and act when the watchdog expires."""
         LOGGER.info("Websocket watchdog expired")
-
-        if asyncio.iscoroutinefunction(self._action):
-            asyncio.create_task(self._action())
-        else:
-            self._loop.call_soon(self._action())
+        schedule_callback(self._action)
 
     def cancel(self) -> None:
         """Cancel the watchdog."""
@@ -213,9 +210,9 @@ class WebsocketClient:
     def __init__(self, api: API) -> None:
         """Initialize."""
         self._api = api
-        self._connect_listeners: list[Callable[..., None]] = []
-        self._disconnect_listeners: list[Callable[..., None]] = []
-        self._event_listeners: list[Callable[..., None]] = []
+        self._connect_callbacks: list[Callable[..., None]] = []
+        self._disconnect_callbacks: list[Callable[..., None]] = []
+        self._event_callbacks: list[Callable[..., None]] = []
         self._loop = asyncio.get_running_loop()
         self._watchdog = Watchdog(self.async_reconnect)
 
@@ -228,15 +225,15 @@ class WebsocketClient:
         return self._client is not None and not self._client.closed
 
     @staticmethod
-    def _add_listener(
-        listener_list: list, callback: Callable[..., Any]
+    def _add_callback(
+        callback_list: list, callback: Callable[..., Any]
     ) -> Callable[..., None]:
-        """Add a listener callback to a particular list."""
-        listener_list.append(callback)
+        """Add a callback callback to a particular list."""
+        callback_list.append(callback)
 
         def remove() -> None:
             """Remove the callback."""
-            listener_list.remove(callback)
+            callback_list.remove(callback)
 
         return remove
 
@@ -283,36 +280,29 @@ class WebsocketClient:
         """Parse an incoming message."""
         if message["type"] == "com.simplisafe.event.standard":
             event = websocket_event_from_payload(message)
-            for listener in self._event_listeners:
-                self._schedule_listener_call(listener, event)
+            for callback in self._event_callbacks:
+                schedule_callback(callback, event)
 
-    def _schedule_listener_call(self, listener: Callable[..., Any], *args: Any) -> None:
-        """Schedule a listener to be called."""
-        if asyncio.iscoroutinefunction(listener):
-            asyncio.create_task(listener(*args))
-        else:
-            self._loop.call_soon(listener, *args)
-
-    def add_connect_listener(self, callback: Callable[..., Any]) -> Callable[..., None]:
-        """Add a listener callback to be called after connecting.
+    def add_connect_callback(self, callback: Callable[..., Any]) -> Callable[..., None]:
+        """Add a callback callback to be called after connecting.
 
         :param callback: The method to call after connecting
         :type callback: ``Callable[..., None]``
         """
-        return self._add_listener(self._connect_listeners, callback)
+        return self._add_callback(self._connect_callbacks, callback)
 
-    def add_disconnect_listener(
+    def add_disconnect_callback(
         self, callback: Callable[..., Any]
     ) -> Callable[..., None]:
-        """Add a listener callback to be called after disconnecting.
+        """Add a callback callback to be called after disconnecting.
 
         :param callback: The method to call after disconnecting
         :type callback: ``Callable[..., None]``
         """
-        return self._add_listener(self._disconnect_listeners, callback)
+        return self._add_callback(self._disconnect_callbacks, callback)
 
-    def add_event_listener(self, callback: Callable[..., Any]) -> Callable[..., None]:
-        """Add a listener callback to be called upon receiving an event.
+    def add_event_callback(self, callback: Callable[..., Any]) -> Callable[..., None]:
+        """Add a callback callback to be called upon receiving an event.
 
         Note that callbacks should expect to receive a WebsocketEvent object as a
         parameter.
@@ -320,7 +310,7 @@ class WebsocketClient:
         :param callback: The method to call after receiving an event.
         :type callback: ``Callable[..., None]``
         """
-        return self._add_listener(self._event_listeners, callback)
+        return self._add_callback(self._event_callbacks, callback)
 
     async def async_connect(self) -> None:
         """Connect to the websocket server."""
@@ -333,8 +323,8 @@ class WebsocketClient:
 
         LOGGER.info("Connected to websocket server")
 
-        for listener in self._connect_listeners:
-            self._schedule_listener_call(listener)
+        for callback in self._connect_callbacks:
+            schedule_callback(callback)
 
         self._watchdog.trigger()
 
@@ -346,8 +336,8 @@ class WebsocketClient:
 
         LOGGER.info("Disconnected from websocket server")
 
-        for listener in self._disconnect_listeners:
-            self._schedule_listener_call(listener)
+        for callback in self._disconnect_callbacks:
+            schedule_callback(callback)
 
     async def async_listen(self) -> None:
         """Start listening to the websocket server."""
@@ -384,8 +374,8 @@ class WebsocketClient:
         finally:
             LOGGER.debug("Listen completed; cleaning up")
 
-            for listener in self._disconnect_listeners:
-                self._schedule_listener_call(listener)
+            for callback in self._disconnect_callbacks:
+                schedule_callback(callback)
 
     async def async_reconnect(self) -> None:
         """Reconnect (and re-listen, if appropriate) to the websocket."""
